var SV = window.SV || {};
SV.AdvancedBBCode = SV.AdvancedBBCode || {};
SV.AdvancedBBCode.EditorConfig = SV.AdvancedBBCode.EditorConfig || [];
SV.AdvancedBBCode.EditorInit = SV.AdvancedBBCode.EditorInit || [];

(/** @param {jQuery} $ jQuery Object */ function ($, window, document, _undefined) {
    "use strict";

    if ('normalizeAfterInsert' in XF.EditorHelpers) {
        return;
    }

    SV.AdvancedBBCode.brSanitize = {
        normalizeBrForEditor: function (content) {
            var asString = typeof content === 'string',
                $fragWrapper;
            if (asString) {
                $fragWrapper = $('<div />').html(content);
            } else {
                $fragWrapper = content;
            }
            var checkNodeMatch = function ($node, elementType) {
                var node = $node.get(0);
                return $node.is(elementType) &&
                    node.className === '' &&
                    !node.hasAttribute('id') &&
                    !node.hasAttribute('style')
            };

            // Workaround editor behaviour that a <br> should not be the first or last child of a <p> tag
            // <p><br>...</p>; editor can delete too many lines
            // <p>...<br></p>; editor can delete too few lines
            // <p>word1 word2<br></p>; editor will delete the space character between word1/word2 when backspacing word2 out
            // Fixed in XF2.2.7

            $fragWrapper.children('p').each(function () {
                if (this.childNodes.length !== 1) {
                    return;
                }
                var $firstChild = $(this.childNodes[0]);
                if (checkNodeMatch($firstChild, 'span')) {
                    $(this).html($firstChild.html());
                }
            });
            $fragWrapper.children('p').each(function() {
                if (this.childNodes.length <= 1) {
                    return;
                }
                var $firstChild = $(this.childNodes[0]);
                if (checkNodeMatch($firstChild, 'br')) {
                    $(this).before($('<p>').append($firstChild));
                }
            });
            $fragWrapper.children('p').each(function() {
                if (this.childNodes.length <= 1) {
                    return;
                }
                var $lastChild = $(this.childNodes[this.childNodes.length - 1]);
                if (checkNodeMatch($lastChild, 'br')) {
                    $lastChild.remove();
                }
            });

            return asString ? $fragWrapper.html() : $fragWrapper;
        },


        normalizeAfterInsert: function(ed) {
            var selected = ed.html.getSelected();
            if (/<br>\s*<\/p>/.test(selected)) {
                XF.EditorHelpers.normalizeBrForEditor(ed.$el);
                // remove the last undo step and replace it with the corrected html version
                ed.undo_index--;
                ed.undo_stack.pop();
                ed.undo.saveStep();
            }
        },

        shimInsertCode: function() {
            var originalFn = XF.EditorHelpers.insertCode;
            XF.EditorHelpers.insertCode = function(ed, type, code) {
                originalFn(ed, type, code);
                XF.EditorHelpers.normalizeAfterInsert(ed);
            }
        },

        shimWrapSelectionText: function() {
            var originalFn = XF.EditorHelpers.wrapSelectionText;
            XF.EditorHelpers.wrapSelectionText = function(ed, before, after, save) {
                originalFn(ed, before, after, save);
                XF.EditorHelpers.normalizeAfterInsert(ed);
            }
        },
    }
    // backport functions
    XF.EditorHelpers.normalizeBrForEditor = SV.AdvancedBBCode.brSanitize.normalizeBrForEditor;
    XF.EditorHelpers.normalizeAfterInsert = SV.AdvancedBBCode.brSanitize.normalizeAfterInsert;

    XF.Element.extend('editor', {
        __backup: {
            'insertContent': 'svAdvancedBbCode_brSanitize_insertContent',
            'normalizePaste': 'svAdvancedBbCode_brSanitize_normalizePaste',
        },
        normalizePaste: function(content) {
            content = this.svAdvancedBbCode_brSanitize_normalizePaste(content);

            var $fragWrapper = $('<div />').html(content);
            $fragWrapper = XF.EditorHelpers.normalizeBrForEditor($fragWrapper);
            content = $fragWrapper.html();
            return content;
        },
        insertContent: function (html, text) {
            // noinspection Annotator
            this.svAdvancedBbCode_brSanitize_insertContent(html, text);
            if (!this.isBbCodeView())
            {
                XF.EditorHelpers.normalizeAfterInsert(this.ed);
            }
        }
    });

    SV.AdvancedBBCode.EditorFirstStart.push(function(editorConfig) {
        SV.AdvancedBBCode.brSanitize.shimInsertCode();
        SV.AdvancedBBCode.brSanitize.shimWrapSelectionText();
    });

    SV.AdvancedBBCode.EditorInit.push(function(config, ed, xfEditor) {
        ed.events.on('paste.after', function () {
            // handle inline copy-paste into a blank line, which results in <p>...<br></p>
            XF.EditorHelpers.normalizeBrForEditor(ed.$el);
        });
    });
})(jQuery, window, document);