var SV = window.SV || {};
SV.AdvancedBBCode = SV.AdvancedBBCode || {};
SV.AdvancedBBCode.EditorConfig = SV.AdvancedBBCode.EditorConfig || [];
SV.AdvancedBBCode.EditorInit = SV.AdvancedBBCode.EditorInit || [];

(/** @param {jQuery} $ jQuery Object */ function ($, window, document, _undefined) {
    "use strict";

    XF.Editor = XF.extend(XF.Editor, {
        __backup: {
            'normalizePaste': 'svAdvancedBbCode_normalizePaste',
        },

        normalizePaste: function(content) {
            var $fragWrapper = $('<div />').html(content),
                rewriteTags = [];

            // restore styling injected by this add-on
            // froala ignores attributes that match `data-fr-*`
            $fragWrapper.find('[data-fr-sv-class]').each(function() {
                var $self = $(this);
                $self.addClass($self.attr('data-fr-sv-class'));
                $self.removeAttr('data-fr-sv-class');
            });

            if (SV.AdvancedBBCode.editorConfig.tags.abbr) {
                $fragWrapper.find('abbr').replaceWith(function () {
                    var title = '' + $(this).attr('title');
                    return '[ABBR' + (title.length ? '="' + title + '"' : '') + ']' + this.innerHTML + "[/ABBR]";
                });
            }

            if (SV.AdvancedBBCode.editorConfig.tags.shimH1) {
                // add default styling for <hr> tags
                var hrStyle = SV.AdvancedBBCode.editorConfig.hrStyle;
                $fragWrapper.find('hr:not(.bbc-hr)')
                    .addClass('bbc-hr' + (hrStyle ? ' bbc-hr--' + hrStyle : ''));

                var tagsToRewrite = $fragWrapper.find('sub, sup, h1, h2, h3, h4');
                if (tagsToRewrite.length !== 0) {
                    rewriteTags = ['SVBBCODE_SUB', 'SVBBCODE_SUP', 'SVBBCODE_H1', 'SVBBCODE_H2', 'SVBBCODE_H3', 'SVBBCODE_H4'];
                    tagsToRewrite.replaceWith(function () {
                        var $replacement = $('<SVBBCODE_' + this.tagName + ' />');
                        $replacement.html(this.innerHTML);
                        $replacement.addClass(this.className);
                        return $replacement;
                    });
                }
            }

            content = $fragWrapper.html();

            content = this.svAdvancedBbCode_normalizePaste(content);

            if (rewriteTags.length !== 0) {
                $fragWrapper = $('<div />').html(content);

                var rewriteTagFilter = rewriteTags.join(', ');
                $fragWrapper.find(rewriteTagFilter).replaceWith(function () {
                    var tag = this.tagName.replace('SVBBCODE_', '');
                    var $replacement = $('<' + tag + ' />');
                    $replacement.html(this.innerHTML);
                    $replacement.addClass(this.className);

                    switch (tag) {
                        case 'H1':
                        case 'H2':
                        case 'H3':
                        case 'H4':
                            if (!$replacement.hasClass('bbc-header')) {
                                // add default styling for h1/h2/h3/h4 tags
                                $replacement.addClass('bbc-header');
                            }
                            break;
                    }
                    return $replacement;
                });

                content = $fragWrapper.html();
            }

            return content;
        }
    });

    SV.AdvancedBBCode.copyNPaste = {
        setupFragWrapper: function (content, normalizeQuoteCharacters, pruneMetadata, inlineClassToCss) {

            if (normalizeQuoteCharacters) {
                content = SV.AdvancedBBCode.copyNPaste.cleanupQuoteCharactersCopyPaste(content);
            }
            // noinspection UnnecessaryLocalVariableJS
            var frag = $.parseHTML(content),
                $fragWrapper = $('<div />').html(frag);

            if (inlineClassToCss) {
                var allowedAttributes = typeof inlineClassToCss === 'object';
                var html = '',
                    iframe = $('<iframe id="ClassToInlineStyleImporter" style="position: absolute;width:0;height:0;border:0;"></iframe>');
                try {
                    iframe.html($fragWrapper.html());
                    iframe.appendTo($('body'));

                    // extract computed set in one go as getComputedStyle is dynamic
                    var list = [];
                    $.uniqueSort(iframe.find('p, span, hr, [class]')).each(function (index, element) {
                        var css = {},
                            computedStyle = window.getComputedStyle(element);
                        if (computedStyle.length === 0) {
                            return;
                        }
                        for (var i = computedStyle.length; i--;) {
                            var nameString = computedStyle[i];
                            if (allowedAttributes && !(nameString in inlineClassToCss)) {
                                continue;
                            }
                            var val = computedStyle.getPropertyValue(nameString);
                            if (val !== '') {
                                css[nameString] = val;
                            }
                        }

                        if (css.length !== 0) {
                            list.push([$(element), css]);
                        }
                    });
                    // apply once we have a static list
                    for(var i=0;i<list.length;i++) {
                        var $e = list[i][0],
                            css = list[i][1];

                        var existingInlineStyle = $e.attr('style')
                        $e.css(css);
                        if (existingInlineStyle) {
                            $e.attr('style', $e.attr('style') + ';' + existingInlineStyle)
                        }
                    }

                    html = iframe.html();
                }
                catch(exception) {
                    console.log(exception);
                }
                iframe.remove();
                if (html !== '') {
                    $fragWrapper = $('<div />').html(html);
                }
            }

            if (pruneMetadata) {
                $fragWrapper.contents().filter(function () {
                    return this.nodeType === Node.COMMENT_NODE;
                }).remove();
                $fragWrapper.find('> xml, > meta, > style, > link, > title').remove();

                $fragWrapper = SV.AdvancedBBCode.copyNPaste.setupFragWrapper($fragWrapper.html().trim());
            }

            return $fragWrapper;
        },
        cleanupQuoteCharactersCopyPaste: function (content) {
            content = content
                // smart quotes
                .replace(/&#x2018;|&#x2019;|[\u2018-\u201B]/gi, "'")
                .replace(/&#x201C;|&#x201D;|[\u201C-\u201F\u301D-\u301F\u2E42]/gi, '"')
                // space characters
                .replace(/&nbsp;|&ensp;|&emsp;|&thinsp;|&#x200[0-9A];|&#x202F;|&#x205F;|&#x3000;|[\u2000-\u200A\u202F\u205F\u3000]/gi, ' ')
            ;
            return content;
        },
        cleanupWordNewlinesCopyPaste:  function ($fragWrapper) {
            // prune span[lanq] to simplify processing. See from older MS Word
            $fragWrapper.find('span[lang], span[style*="mso-ansi-language"], span[style*="mso-fareast-language"]').each(function(index, element) {
                var $self = $(element);
                $self.replaceWith($self.html());
            });

            $fragWrapper.find('div hr').each(function(index, element) {
                var $self = $(element);
                $self.unwrap();
            });

            // MsoNoSpacing ==  "No spacing" paragraph style
            // MsoNormal ==  Normal paragraph style, extra line after each paragraph
            // However, this is derived from the line-height via the css which can sometimes change things.
            $fragWrapper.children('p.MsoNoSpacing,p.MsoNormal').each(function(index, element) {
                var $self = $(element),
                    $e = $('<span />');
                $e.attr('style', $self.attr('style'));
                $self.contents().each(function() {
                    if(this.nodeType === Node.COMMENT_NODE || this.tagName === 'O:P') {
                        $(this).remove();
                    }
                });
                $e.html($self.html());
                $e.addClass(element.className);

                SV.AdvancedBBCode.copyNPaste.appendBrForSpanRewrite($e, 0);

                var marginBottomAttr = '' + element.style.marginBottom;
                var extraAppendOnEmpty;
                if (marginBottomAttr !== '') {
                    var bottomPadding = SV.AdvancedBBCode.copyNPaste.getAttrLineHeightPercent(element, marginBottomAttr, 0);
                    if (bottomPadding > 0.1) {
                        element.style.marginBottom = '';
                        element.style.marginTop = '';
                        extraAppendOnEmpty = 1;
                    } else {
                        extraAppendOnEmpty = 0;
                    }
                } else {
                    extraAppendOnEmpty = $e.hasClass('MsoNormal');
                }

                if (extraAppendOnEmpty && $e.length > 0) {
                    $e.append($('<br>'));
                }

                $self.replaceWith($e);
            });

            return $fragWrapper;
        },
        appendBrForSpanRewrite: function($e, extraAppendOnEmpty = 0, appendDefault = 1) {
            // append a newline if one isn't in the last child of the paragraph (skipping whitespace)
            var
                append  = appendDefault,
                childNodes = $e.get(0).childNodes;
            for(var i = childNodes.length -1; i >= 0; i--) {
                var node = childNodes[i];
                if (node.nodeType === Node.TEXT_NODE && (!node.nodeValue || /^[\s\n]+$/.test(node.nodeValue))) {
                    continue;
                }

                append = node.tagName !== 'BR';
                break;
            }
            if (append) {
                if (extraAppendOnEmpty && childNodes.length > 0) {
                    $e.append($('<br>'));
                }
                $e.append($('<br>'));
            }
        },
        getAttrLineHeightPercent: function(element, attr, defaultVal) {
            // if line-height is in px, check the font-size to compute the percentage line-height
            // otherwise default to 100%
            // note; not all paragraph styles add an explicit line-height, and
            // projecting the default style into inline attributes returns px values

            attr = attr ? attr : defaultVal + '%';

            var lineHeight = defaultVal,
                match = attr.match(/^(\d+(?:\.\d+)?)px$/)
            if (match && match[1]) {
                lineHeight = parseFloat(match[1]);
                attr = '' + element.style.fontSize;
                match = attr.match(/^(\d+)px$/)
                if (match && match[1]) {
                    var fontSize = parseFloat(match[1]);
                    lineHeight = ((100 * lineHeight) / fontSize) | 0;
                }
            } else {
                match = attr.match(/^(\d+)\%$/);
                if (match && match[1]) {
                    lineHeight = match[1] | 0;
                }
            }

            return lineHeight;
        },
        cleanupLibreOfficeNewlinesCopyPaste: function ($fragWrapper, lineHeightThreshold) {

            // OpenOffice supports inserting an image as a horizontal ruler, which LibreOffice supports
            $fragWrapper.find('p > img[src*="gallery/rulers"]').each(function(index, element) {
                var $self = $(element);
                if (/file:\/\/.*\/gallery\/rulers\/.*.gif/i.test($self.attr('src'))) {

                    var $e = $('<hr />');
                    $e.attr('style', $self.attr('style'));
                    $self.replaceWith($e);
                }
            });

            // a <p> with a line-height of >= 1 is effectively a span with a bunch of <br> on the end
            // except if it is <p><br><p> which is a blank line
            $fragWrapper.children('p').each(function(index, element) {
                var lineHeight = SV.AdvancedBBCode.copyNPaste.getAttrLineHeightPercent(element, element.style.lineHeight, 100);
                if (lineHeight) {
                    var $self = $(element),
                        $e = $('<span />');

                    element.style.lineHeight = '';
                    $e.attr('style', $self.attr('style'));
                    $e.html($self.html());

                    SV.AdvancedBBCode.copyNPaste.appendBrForSpanRewrite($e, 0);

                    // "Text Body" paragraph formatting threshold, varies between OpenOffice and LibreOffice
                    if (lineHeight >= lineHeightThreshold && lineHeight < 200) {
                        lineHeight = 200;
                    }
                    var lineHeightPercentage = Math.floor(lineHeight / 100) | 0;
                    while (lineHeightPercentage > 1) {
                        lineHeightPercentage--;
                        $e.append($('<br>'));
                    }


                    $self.replaceWith($e);
                }
            });

            $fragWrapper.children('hr').each(function(index, element) {
                var lineHeight = SV.AdvancedBBCode.copyNPaste.getAttrLineHeightPercent(element, element.style.lineHeight, 100);
                if (lineHeight) {
                    var $self = $(element);

                    // round up if it isn't small enough
                    if (lineHeight >= 115 && lineHeight < 200) {
                        lineHeight = 200;
                    }

                    var lineHeightPercentage = Math.floor(lineHeight / 100) | 0;
                    while (lineHeightPercentage > 1) {
                        lineHeightPercentage--;
                        $self.after($('<br>'));
                    }
                }
            });

            return $fragWrapper;
        },
        cleanupOpenOfficeNewlinesCopyPaste: function ($fragWrapper) {

            // cleanupLibreOfficeNewlinesCopyPaste rewrites p => span, and handles newlines

            // Increase spacing; margin-top/margin-bottom - 0.1cm - 0.31cm - 1 line
            // Text Body paragraph formatting; margin-bottom:0.21cm - 1 line
            // projecting the default style into inline attributes returns px values
            $fragWrapper.children('span[style*="margin"]').each(function(index, element) {
                var marginBottomAttr = '' + element.style.marginBottom;
                if (marginBottomAttr === '') {
                    return;
                }
                var lineHeight = SV.AdvancedBBCode.copyNPaste.getAttrLineHeightPercent(element, marginBottomAttr, 0);
                if (lineHeight >= 115 / 2) {
                    element.style.marginBottom = '';
                    element.style.marginTop = '';
                    $(element).append($('<br>'));
                }
            });

            return $fragWrapper;
        },
        cleanupHtmlCopyPaste: function ($fragWrapper, detectLineEditor, isGoogleDocs) {
            var nodes;

            // handle discord code blocks
            nodes = $fragWrapper.find('pre > code.hljs');
            nodes.each(function(index, element){
                var $self = $(element),
                    $parent = $self.parent(),
                    $inner = $('<span />');

                // convert to a [code=XX][/code] block
                var match = $self.attr("class").match(/.*\s+hljs\s+([^\s]+)$/i);
                var type = match ? "=" + match[1] : '';

                $inner.attr('style', $self.attr('style') + ';' + $parent.attr('style'));
                $inner.text($self.text());

                // avoid formatting being pulled across the bb-code header tags which cause round-tripping issues
                var $outer = $('<p data-xf-p="1" />');
                $outer.append('[code'+type+']\n');
                $outer.append($inner);
                $outer.append('[/code]');

                $parent.replaceWith($outer);
            });
            // handle discord/XF inline code blocks
            nodes = $fragWrapper.find('code.inline, code.bbCodeInline');
            nodes.each(function(index, element){
                var $self = $(element);

                // convert to a [icode][/icode] block
                var $inner = $("<span />");
                $inner.attr('style', $self.attr('style'));
                $inner.text($self.text());

                // avoid formatting being pulled across the bb-code header tags which cause round-tripping issues
                var $outer = $('<p data-xf-p="1" />');
                $outer.append('[icode]');
                $outer.append($inner);
                $outer.append('[/icode]');

                $self.replaceWith($outer);
            });
            // handle discord inline spoiler blocks
            nodes = $fragWrapper.find('span[class*="spoilerText"][aria-expanded] > span[aria-hidden], span.bbCodeInlineSpoiler, span.bbc-spoiler');
            nodes.each(function(index, element){
                var $self = $(element);

                $self.removeClass('bbc-spoiler bbc-spoiler-instant');
                // wrap with [ispoiler]...[/ispoiler] block, but leave the original div alone to preserve formatting
                $self.prepend("[ispoiler]");
                $self.append("[/ispoiler]");
            });

            // convert XF usernames into actual @username text
            nodes = $.uniqueSort($fragWrapper.find('.message-name .username[itemprop="name"], .username[itemprop="name"]'))
            nodes.each(function(index, element){
                var $self = $(element),
                    $parent = $self.parent(),
                    text = $self.text(),
                    $e = $('<span />');

                text = text.replace(/^@/, '');
                var userId = $self.data('user-id') | 0;
                var prefix = userId ? '@' : '';
                text = prefix + text + ' ';

                $e.text(text);

                if ($parent.hasClass('message-name')) {
                    $parent.replaceWith($e);
                } else {
                    $self.replaceWith($e);
                }
            });

            nodes = $fragWrapper.find('h1, h2, h3, h4, h5, h6')
            nodes.each(function(index, element){
                var $self = $(element),
                    tagName = element.tagName;
                if (tagName === 'H5' || tagName === 'H6') {
                    tagName = 'H4';
                }

                // convert header to bb-code, as editor behaviour is bad
                var $inner = $('<span />');
                $inner.attr('style', $self.attr('style'));
                // due to wonky XF editor behaviour trim leading newlines
                $inner.html($self.html().replace(/^\n+/, ''));

                // avoid formatting being pulled across the bb-code header tags which cause round-tripping issues
                var $outer = $('<p data-xf-p="1" />');
                $outer.append('[' + tagName + ']');
                $outer.append($inner);
                $outer.append('[/' + tagName + ']');

                $self.replaceWith($outer);
            });

            if (SV.AdvancedBBCode.editorConfig.pasteQuotesAsBbCode) {
                nodes = $fragWrapper.find('blockquote');
                nodes.each(function (index, element) {
                    var $self = $(element),
                        $inner = $("<span class='blockquote-span' />");

                    $inner.attr('style', $self.attr('style'));
                    $inner.html($self.html());

                    var $outer = $('<p data-xf-p="1" />');
                    $outer.append('[QUOTE]\n');
                    $outer.append($inner);
                    $outer.append('[/QUOTE]');

                    $self.replaceWith($outer);
                });
            }

            // convert apple-spaces <span className="Apple-converted-space">&nbsp;</span>
            nodes = $fragWrapper.find('span.Apple-converted-space');
            nodes.each(function(index, element) {
                var $self = $(element),
                    html = $self.html();
                if (html === '&nbsp;') {
                    $self.replaceWith(' ');
                }
            });

            nodes = $fragWrapper.find('p, span, div, blockquote, code');
            // possible google docs paste. will be a list of <p> with a collection of <span> or a collection of <span>
            nodes.each(function(index, element){
                var $self = $(element),
                    wrap = [],
                    attr,
                    bbCode;

                // font-weight: >700 => <b>
                attr = element.style.fontWeight;
                element.style.fontWeight = "";
                if (Number(attr) >= 700 || attr === 'bold') {
                    wrap.push('strong');
                }

                if (!$self.parent('pre').length) {
                    attr = element.style.whiteSpace;
                    if (attr === '' && ($self.is('blockquote') || $self.hasClass('blockquote-span'))) {
                        // fetch from parent
                        attr = $self.css('white-space');
                        if (attr === '') {
                            var $parents = $self.parent('[style*="white-space:"]').first();
                            if ($parents.length) {
                                attr = $parents.get(0).style.whiteSpace;
                            }
                        }
                    }
                    if (/^pre(-.*)?$|^break-spaces$/i.test(attr)) {
                        element.style.whiteSpace = '';
                        var patchTextFragment = function(node) {
                            if (!node.hasChildNodes()) {
                                return;
                            }
                            var children = node.childNodes;
                            // node list is "live" so, need to bump i based on adding/removing nodes!
                            for (let i = 0; i < children.length; i++) {
                                var childNode = children[i];

                                if (childNode.hasChildNodes()) {
                                    patchTextFragment(childNode);
                                } else if (childNode.nodeType === Node.TEXT_NODE && childNode.nodeValue) {
                                    var textFragments = childNode.nodeValue.split(/\r?\n/)
                                    if (textFragments.length > 1) {
                                        textFragments.forEach(function (val, i2) {
                                            var text = document.createTextNode(val);
                                            node.insertBefore(text, childNode);
                                            i++;
                                            if (i2 + 1 < textFragments.length) {
                                                var br = document.createElement("br")
                                                node.insertBefore(br, childNode);
                                                i++;
                                            }

                                        });
                                        node.removeChild(childNode);
                                        i--;
                                    }
                                }
                            }
                        }
                        element.normalize();
                        patchTextFragment(element);
                    }
                }

                // font-style:italic (or oblique) => <em>
                attr = element.style.fontStyle;
                element.style.fontStyle = "";
                if (/^italic|^oblique/i.test(attr)) {
                    wrap.push('em');
                }

                // text-decoration: line-through/underline => <s>/<u>
                attr = element.style.textDecoration;
                element.style.textDecoration = "";
                if (attr === 'line-through') {
                    wrap.push('s');
                } else if (attr === 'underline') {
                    wrap.push('u');
                }

                // vertical-align: super/syb => <sup>/<sub>
                attr = element.style.verticalAlign;
                element.style.verticalAlign = "";
                if (attr === 'super') {
                    wrap.push('sup');
                } else if (attr === 'sub') {
                    wrap.push('sub');
                }

                // Editor doesn't preserve 'text-align' so rewrite into bb-code
                // text-align: center/justify/end => [centre]/[justify]/[right] bb-code
                attr = element.style.textAlign;
                element.style.textAlign = "";
                bbCode = '';
                switch(attr) {
                    case 'center':
                        bbCode = 'center';
                        break;
//                    case 'justify':
//                    case 'justify-all':
//                        bbCode = 'justify';
//                        break;
                    case 'end':
                    case 'right':
                        bbCode = 'right';
                        break;
                }
                if (bbCode !== '') {
                    $self.prepend('<span>[' +  bbCode + ']</span>');
                    $self.append('<span>[/' +  bbCode + ']</span>');
                }


                // 'color:.*' => <color>
                // todo...

                if (wrap.length) {
                    var wrapper = $self;
                    $.each(wrap, function(k, v){
                        var attr, e;
                        if (Array.isArray(v)) {
                            e = v[0];
                            attr = v[1];
                        } else {
                            e = v;
                            attr = "";
                        }
                        wrapper = wrapper.wrapInner("<" + e + " "+ attr +  "></" + e + ">");
                    });
                }

                //element.style = {};
            });

            // froala ignores attributes that match `data-fr-*`
            // stash classes starting with "bbc-" into an attribute
            $fragWrapper.find('[data-fr-sv-class]').removeAttr('data-fr-sv-class');
            $fragWrapper.find('[class*="bbc-"]').each(function(){
                var results = this.className.match(/(bbc-[^\s]+)/gi);
                if (results.length !== 0) {
                    $(this).attr('data-fr-sv-class', results.join(' '));
                }
            });

            if (isGoogleDocs) {
                // Google Docs can output <p style="margin-top:1pt,margin-bottom:1pt"></p><br/> which results in unexpected doublespacing
                $fragWrapper.find('p[style*="margin-top:1pt"]').each(function () {
                    if (this.style.marginTop === '1pt' && this.style.marginBottom === '1pt' &&
                        (!this.nextElementSibling || this.nextElementSibling.tagName === 'BR')) {
                        this.style.marginTop = 0;
                        this.style.marginBottom = 0;
                    }
                });
            }

            if (!detectLineEditor) {
                return $fragWrapper;
            }

            // Detect "<p>...</p><p><br></p><p>...</p>" line-based editing and prevent XF injecting newlines
            // only p children, and at least one is just <p><br><p>, no p nodes which are empty
            var parentNode = $fragWrapper.get(0),
                childNodes = parentNode.childNodes;
            if (childNodes.length > 1) {
                var valid = true,
                    hasBr = false,
                    pruneList = [];
                for (var i = childNodes.length - 1; i >= 0; i--) {
                    var node = childNodes[i];
                    if (node.nodeType === Node.COMMENT_NODE) {
                        pruneList.push(node);
                        continue;
                    }
                    if (node.nodeType === Node.TEXT_NODE && (!node.nodeValue || /^[\s\n]+$/.test(node.nodeValue))) {
                        pruneList.push(node);
                        continue;
                    }
                    if (node.tagName === 'STYLE' ||
                        node.tagName === 'META' ||
                        node.tagName === 'LINK' ||
                        node.tagName === 'TITLE' ||
                        node.tagName === 'XML') {
                        pruneList.push(node);
                        continue;
                    }

                    var validChild,
                        childNodes2 = node.childNodes;
                    if (node.tagName !== 'P' && node.tagName !== 'HR') {
                        validChild = false;
                    } else {
                        validChild = false;
                        // check that this is node isn't empty, and flag if we have a br
                        for (var i2 = childNodes2.length - 1; i2 >= 0; i2--) {
                            var childNode = childNodes2[i2];
                            if (childNode.nodeType === Node.COMMENT_NODE) {
                                continue;
                            }
                            if (childNode.nodeType === Node.TEXT_NODE && (!childNode.nodeValue || /^[\s\n]+$/.test(childNode.nodeValue))) {
                                continue;
                            }

                            if (childNode.tagName === 'BR') {
                                hasBr = true;
                            }

                            // has some text or nodes
                            validChild = true;
                            break;
                        }
                    }

                    if (!validChild) {
                        valid = false;
                        break;
                    }
                }
                if (valid && hasBr) {
                    // `data-xf-p` tells XF not to double the <p>'s,
                    // however, need to remove non-block nodes between <p> tags or XF will wrap them in a <p> tag
                    for (i in pruneList) {
                        parentNode.removeChild(pruneList[i]);
                    }
                    $fragWrapper.children('p').attr('data-xf-p', '1');
                }
            }

            return $fragWrapper;
        },
    }

/*
    SV.AdvancedBBCode.EditorConfig.push(function (editorConfig, config, ed, xfEditor) {
        config.pasteAllowedStyleProps = config.pasteAllowedStyleProps || [];
        [
            //'color','font-family',
            //'font-size','text-indent','text-align'
        ].forEach(function(element) { config.pasteAllowedStyleProps.push(element); });
        if ('pasteDeniedAttrs' in config) {
            for (var i = 0; i < config.pasteDeniedAttrs.length; i++) {
                if (config.pasteDeniedAttrs[i] === 'style') {
                    config.pasteDeniedAttrs.splice(i, 1);
                    break;
                }
            }
        } else {
            config.pasteDeniedAttrs = ['id','class'];
        }
    });
*/

    SV.AdvancedBBCode.EditorInit.push(function(config, ed, xfEditor) {
        ed.events.on('paste.beforeCleanup', function(content)
        {
            // note; content is html in string form
            var $fragWrapper,
                detectLineEditor = false,
                isGoogleDocs = false,
                normalizeQuoteChars = SV.AdvancedBBCode.editorConfig.normalizeQuoteChars;
            // XF cleanup occurs in 'paste.beforeCleanup', and word paste cleanup needs to occur before then
            // paste.wordPaste triggers to later to be used
            if (/class="?Mso|class='?Mso|class="?Xl|class='?Xl|class=Xl|style="[^"]*\bmso-|style='[^']*\bmso-|w:WordDocument/i.test(content)) {
                $fragWrapper = SV.AdvancedBBCode.copyNPaste.setupFragWrapper(content, normalizeQuoteChars, true, {
                    // allowed style attributes to inline
                    'margin-bottom':1,
                    'margin-top':1
                });
                $fragWrapper = SV.AdvancedBBCode.copyNPaste.cleanupWordNewlinesCopyPaste($fragWrapper);
            } else if (/<meta[^>]+?LibreOffice/i.test(content)) {
                $fragWrapper = SV.AdvancedBBCode.copyNPaste.setupFragWrapper(content, normalizeQuoteChars, true, true);
                $fragWrapper = SV.AdvancedBBCode.copyNPaste.cleanupLibreOfficeNewlinesCopyPaste($fragWrapper, 115);
            } else if (/<meta[^>]+?OpenOffice/i.test(content)) {
                $fragWrapper = SV.AdvancedBBCode.copyNPaste.setupFragWrapper(content, normalizeQuoteChars, true, true);
                $fragWrapper = SV.AdvancedBBCode.copyNPaste.cleanupLibreOfficeNewlinesCopyPaste($fragWrapper, 141);
                $fragWrapper = SV.AdvancedBBCode.copyNPaste.cleanupOpenOfficeNewlinesCopyPaste($fragWrapper);
            } else {
                $fragWrapper = SV.AdvancedBBCode.copyNPaste.setupFragWrapper(content, normalizeQuoteChars, false);

                // try detect google-docs
                var $googleDocRoot = $fragWrapper.children('[id*="docs-internal-guid-"]');
                isGoogleDocs = $googleDocRoot.length === 1 && $fragWrapper.children('p,hr').length === 0 && $googleDocRoot.children('p,hr,br').length > 0;
                if (isGoogleDocs) {
                    $googleDocRoot.children().unwrap();
                }

                detectLineEditor = true;
            }

            $fragWrapper = SV.AdvancedBBCode.copyNPaste.cleanupHtmlCopyPaste($fragWrapper, detectLineEditor, isGoogleDocs);

            content = $fragWrapper.html();

            return content;
        }, true);

        var copyPasteContainerSelector = xfEditor.$target.data('debug-copy-paste-output'),
            $copyPasteContainer = copyPasteContainerSelector ? $(copyPasteContainerSelector) : null;
        if ($copyPasteContainer !== null && $copyPasteContainer.length !== 0) {
            var $copyNPasteDebugging = $('input[type="checkbox"][name="showCopyAndPasteDebugging"]');
            $copyNPasteDebugging.change(function() {
                if (this.checked) {
                    $copyPasteContainer.show();
                } else {
                    $copyPasteContainer.hide();
                }
            });

            ed.events.on('paste.beforeCleanup', function(content)
            {
                if (/^\s+$/i.test(content)) {
                    $copyPasteContainer.hide();
                    $copyPasteContainer.empty();
                } else {
                    $copyPasteContainer.empty();
                    $copyPasteContainer.append('<hr>');
                    $copyPasteContainer.append($("<pre style='word-wrap:break-word;white-space:pre-wrap;' />").text(content));
                    $copyPasteContainer.append('<hr>');
                    if ($copyNPasteDebugging.length === 0 || $copyNPasteDebugging.get(0).checked) {
                        $copyPasteContainer.show();
                    }
                }

                return content;
            }, true);

            function checkContents() {
                var text = ed.opts.wordCounterGetText
                    ? ed.opts.wordCounterGetText()
                    : (ed.el.innerText || "");
                if (/^\s+$/i.test(text)) {
                    $copyPasteContainer.hide();
                    $copyPasteContainer.empty();
                }
            }

            ed.events.on("contentChanged", checkContents);
        }
    });

})(jQuery, window, document);