var SV = window.SV || {};
SV.AdvancedBBCode = SV.AdvancedBBCode || {};
SV.AdvancedBBCode.EditorButtons = SV.AdvancedBBCode.EditorButtons || {};
SV.AdvancedBBCode.EditorFirstStart = SV.AdvancedBBCode.EditorFirstStart || [];

(/** @param {jQuery} $ jQuery Object */ function ($, window, document, _undefined) {
    "use strict";

    SV.AdvancedBBCode.DialogForm = XF.Element.newHandler({
        options: {
            output: '[name="bb_code_output"]',
            bbCode: null,
            oldFormat: null,
            newLines: true,
            asJson: false,
            singleOption: false
        },

        $output: null,

        init: function () {
            if (!this.options.bbCode) {
                console.error('Invalid bb code provided.');
                return;
            }

            var $output = this.$target.find(this.options.output);
            if (!$output.length) {
                console.error('Missing output field.');
                return;
            }
            this.$output = $output;

            var self = this;
            this.$target.find(':input').each(function () {
                var $input = $(this);
                if ($input.is('[data-option-name]') || $input.is('[data-bb-value]')) {
                    $input.on('keypress change paste input', XF.proxy(self, 'generateOutput'));
                }
            });

            this.$target.find('[data-xf-click="sv-advanced-bb-code-pack--preview-click"]').on('click', XF.proxy(this, 'generateOutput'));
            this.$target.on('submit', XF.proxy(this, 'generateOutput'));
        },

        generateOutput: function (e) {
            var self = this,
                jsonBlob = {},
                asJson = this.options.asJson,
                options = [],
                values = [],
                inputsScanned = [],
                showErrors = e.type === 'submit';

            this.$target.find(':input:visible, input[type=hidden]').each(function () {
                var $input = $(this),
                    inputName = $input.attr('name'),
                    id, label, text;

                if (!$input.is(self.options.output) && !$input.is('[type="submit"]')) {
                    if ($input.is('[data-option-name]')) {
                        var optionName = $input.data('option-name'),
                            optionType = $input.data('option-type') || 'string',
                            nameless = $input.data('option-nameless') === 'true';

                        if ($.inArray(inputName, inputsScanned) === -1) {
                            inputsScanned.push(inputName);

                            if (!optionName) {
                                console.error('Invalid option provided.');
                                return;
                            }

                            var optionValue = self.valueFromInput(self.$target, $input, 'option');
                            if (optionValue) {
                                if (asJson) {
                                    optionValue = self.getFormattedValue(optionType, optionValue, $input, true);
                                    if (optionValue !== null) {
                                        jsonBlob[optionName.replace('-', '_')] = optionValue;
                                    }
                                } else {
                                    options.push(self.optionStr(optionName, optionValue, optionType, $input, nameless));
                                }
                            }
                        }
                    } else if (self.getFormattedValue('bool', $input.data('bb-value'), $input)) {
                        var parseValue = true;
                        // skip the last field-addr that has not been cloned
                        var $container= $input.closest('.formRow');
                        if ($container.is('[data-increment-format]')) {
                            var fieldAdder = $container.data('xfElementHandlers')['field-adder'];
                            if (!fieldAdder.cloned) {
                                parseValue = false;
                            }
                        }

                        if (parseValue) {
                            var isRequire = self.getFormattedValue('bool', $input.data('bb-require'), $input),
                                requireIfNot = $input.data('bb-require-if-not') || null,
                                requirePhrase = self.getFormattedValue('str', $input.data('bb-require-phrase'), $input) || null,
                                valueType = $input.data('bb-type') || 'string',
                                value = self.valueFromInput(self.$target, $input, 'bb_code');

                            if (!value && showErrors) {
                                if (isRequire) {
                                    id = $input.attr('id');
                                    if (id) {
                                        label = $("label[for='" + id + "']");
                                        if (label.length === 1) {
                                            text = label.text().trim();
                                            if (text) {
                                                inputName = text;
                                            }
                                        }
                                    }

                                    var showErrorForCurrentInput = true,
                                        otherInputName = '';
                                    if (typeof requireIfNot === 'string') {
                                        var otherElement = self.$target.find(requireIfNot);
                                        if (otherElement.length) {
                                            showErrorForCurrentInput = !self.getFormattedValue(self.valueFromInput(self.$target, $input, 'bb_code'));
                                            if (showErrorForCurrentInput) {
                                                otherInputName = otherElement.attr('name');
                                                id = otherElement.attr('id');
                                                if (id) {
                                                    label = $("label[for='" + id + "']");
                                                    if (label.length === 1) {
                                                        text = label.text().trim();
                                                        if (text) {
                                                            otherInputName = text;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }

                                    if (showErrorForCurrentInput) {
                                        if (!requirePhrase) {
                                            requirePhrase = 'please_enter_value_for_required_field_x';
                                        }

                                        XF.overlayMessage(XF.phrase('oops_we_ran_into_some_problems'), XF.phrase(requirePhrase, {
                                            '{field}': inputName,
                                            '{otherField}': otherInputName
                                        }));

                                        e.stopImmediatePropagation();
                                        e.preventDefault();

                                        return;
                                    }
                                }
                            }

                            value = self.valueStr(value, valueType, $input);
                            if (value) {
                                if (asJson) {
                                    optionValue = self.getFormattedValue(valueType, value, $input, true);
                                    if (optionValue !== null) {
                                        if (!jsonBlob['values']) {
                                            jsonBlob['values'] = [];
                                        }
                                        jsonBlob['values'].push(optionValue);
                                    }
                                } else {
                                    values.push(value);
                                }
                            }
                        }
                    }
                }
            });

            var tag = XF.htmlspecialchars(this.options.bbCode).toString().toUpperCase();
            var bbCode = '',
                bbCodeStart = '',
                bbCodeEnd = '',
                respectPreviousCursor = false;
            if (asJson) {
                var json = JSON.stringify(jsonBlob, null, 2);
                respectPreviousCursor = false;
                bbCodeStart = '[' + tag + ' json=1]<br/>\n';
                bbCodeEnd = '<br/>\n[/' + tag + ']';
                bbCode = bbCodeStart + json + bbCodeEnd;
            }
            else if (options.length || values.length) {
                if (options.length) {
                    options = $.trim(options.join(this.options.oldFormat ? '|' : ' '));
                    if (options) {
                        var optionPrefix = ' ';
                        if (this.options.oldFormat || this.options.singleOption) {
                            optionPrefix = '=';
                        }
                        options = optionPrefix + options;
                    }
                }

                if (options || values.length) {
                    var padding = this.options.newLines ? '<br/>\n' : '';
                    respectPreviousCursor = values.length === 0;
                    bbCodeStart = '[' + tag + options + ']' + padding;
                    bbCodeEnd = padding + '[/' + tag + ']';

                    if (values.length) {
                        values = values.join(this.options.newLines ? '<br/>\n' : ' ');
                        var caretPlaceHolderTpl = '<span data-caret-placeholder="true"></span>',
                            re = new RegExp(caretPlaceHolderTpl, "gi"),
                            hasCaretPlaceholder = (values.match(re) || []).length !== 0;

                        if (hasCaretPlaceholder) {
                            respectPreviousCursor = true;
                            var tmpPlaceholder = '<span id="' + Math.random().toString(36).substr(10, 20) + '"></span>';
                            var split = values
                                .replace(caretPlaceHolderTpl, tmpPlaceholder)
                                .replace(re, '')
                                .split(tmpPlaceholder);
                            values = '';
                            bbCodeStart = bbCodeStart + split[0];
                            bbCodeEnd = split[1] + bbCodeEnd;
                        }
                    } else {
                        values = padding;
                    }

                    bbCode = bbCodeStart + values + bbCodeEnd
                }
            } else {
                respectPreviousCursor = true;
                bbCodeStart = '[' + tag + ']<br/>\n';
                bbCodeEnd = '<br/>\n[/' + tag + ']';
                bbCode = bbCodeStart + bbCodeEnd;
            }

            this.$output.data('bb-code-start', bbCodeStart);
            this.$output.data('bb-code-end', bbCodeEnd);
            this.$output.data('bb-code-respect-cursor', respectPreviousCursor ? 'true' : 'false');
            this.$output.val(bbCode);
        },

        valueFromInput: function (form, input, inputType) {
            if (!inputType) {
                console.error('No input type provided.');
                return;
            }

            var self = this,
                inputName = input.attr('name'),
                valueType = null,
                delimiter = null;

            if (inputType === 'option') {
                valueType = input.data('option-type');
                delimiter = input.data('option-delimiter');
            } else if (inputType === 'bb_code') {
                valueType = input.data('bb-type');
                delimiter = input.data('bb-delimiter');
            }

            valueType = valueType || 'string';
            delimiter = delimiter || ',';

            if (valueType === 'time') {
                var name = input.attr('name');
                // assemble the parts
                var ymd = form.find('[name="' + name + '[ymd]' + '"]').val().trim();
                if (ymd === '') {
                    // todo fix-me
                    input.val('');
                    return '';
                }
                var zeroPad = function (n) {
                    return n.length >= 2 ? n : new Array(2 - n.length + 1).join('0') + n;
                };
                var hh = form.find('[name="' + name + '[hh]' + '"]').val();
                var mm = form.find('[name="' + name + '[mm]' + '"]').val();
                var ss = form.find('[name="' + name + '[ss]' + '"]').val();
                var tz = form.find('[name="' + name + '[tz]' + '"]').val();

                input.val(ymd + ' ' + zeroPad(hh) + ':' + zeroPad(mm) + ':' + zeroPad(ss) + ' ' + tz);

                return input.val();
            }

            if (input.is('[type="text"]') || input.is('textarea') || input.is('[type="number"]')) {
                return input.val();
            } else if (input.is('[type="radio"]')) {
                var checkedRadio = form.find('[name="' + inputName + '"]:checked');
                if (checkedRadio.length === 1) {
                    return checkedRadio.val();
                }
            } else if (input.is('[type="checkbox"]')) {
                var checkedBoxes = form.find('[name="' + inputName + '"]:checked');
                if (checkedBoxes.length) {
                    var checkedValues = [];
                    checkedBoxes.each(function () {
                        var checkbox = $(this);
                        checkedValues.push(self.getFormattedValue(valueType, checkbox.val()));
                    });

                    return checkedValues.join(delimiter);
                }
            } else if (input.is('select')) {
                var isMultiple = input.prop('multiple');
                if (isMultiple) {
                    var selectedValues = [];
                    $input.find('option:selected').each(function () {
                        var radio = $(this);
                        selectedValues.push(self.getFormattedValue(valueType, radio.val()));
                    });

                    return selectedValues.join(delimiter);
                } else {
                    return input.val();
                }
            }

            return null;
        },

        getFormattedValue: function (type, value, input, trusted) {
            trusted = trusted || false;
            value = $.trim(value);
            var originalType = typeof value;
            var inputName = null;
            if (input) {
                inputName = input.attr('name');
            }

            switch (type) {
                case 'time':
                case 'str':
                case 'string':
                    if (input && input.attr('data-value') === value) {
                        return null;
                    }
                    if (originalType !== 'string') {
                        value = String(value);
                    }
                    break;

                case 'bbCode':
                    value = this.getFormattedValue('str', value, input, trusted).toString();
                    trusted = true;
                    // replace \n with <br>\n>
                    value = value.replaceAll("\n", "<br>\n");
                    break;

                case 'int':
                case 'number':
                case 'seconds':
                    if (input && input.attr('data-value') === value) {
                        return null;
                    } else if (originalType !== 'number') {
                        value = Number(value);
                        if (isNaN(value)) {
                            value = 0;
                        }
                        if (type === 'seconds') {
                            value = value + 's';
                        }
                    }
                    break;

                case 'bool':
                case 'boolean':
                    if (originalType !== 'boolean') {
                        switch (value) {
                            case 'true':
                            case 'yes':
                            case 'on':
                            case '1':
                                value = true;
                                break;

                            default:
                                value = false;
                        }
                        trusted = true;
                    }
                    break;

                case 'width':
                case 'height':
                    if (input) {
                        var unitInput = input.parent().find('[name="' + inputName + '_unit"]');
                        if (!unitInput.length) {
                            console.error('No unit input available for ' + type + ' input');
                            return;
                        }

                        var unit = unitInput.val();
                        switch (unit) {
                            case 'px':
                            case 'pixels':
                                unit = 'px';
                                break;

                            case '%':
                            case 'percentage':
                                unit = '%';
                                break;

                            case 'auto':
                                value = 'auto';
                                unit = null;
                                break;

                            default:
                                value = null;
                                unit = null;
                                break;
                        }

                        if (unit !== null) {
                            value = this.getFormattedValue('int', value, null);
                            if (value !== null) {
                                if (input && unitInput.attr('data-value') === unit && input.attr('data-value') === value) {
                                    value = null;
                                } else {
                                    value = value.toString() + unit;
                                }
                            }
                        } else if (unitInput.attr('data-value') === 'auto') {
                            value = null;
                        }
                        trusted = true;
                    }
                    break;

                case 'slide':
                case 'slide_header':
                    var tag = input.data('bb-type').toUpperCase();
                    var body = value ? this.getFormattedValue('bbCode', value, input) : '';
                    var oldBraceType = input.data('brace-type') === 'old';
                    var openBrace = oldBraceType ? '{' : '[';
                    var closeBrace = oldBraceType ? '}' : ']' ;
                    if (tag === 'SLIDE') {
                        body += '<br/><span data-caret-placeholder="true"></span>';
                    }

                    value = openBrace + tag + closeBrace + '<br/>\n' + body + '<br/>\n' + openBrace + '/' + tag + closeBrace;
                    trusted = true;
                    break;
            }

            if (!trusted) {
                value = XF.htmlspecialchars(value);
            }

            return value;
        },

        optionStr: function (name, value, type, input, nameless, trusted) {
            type = type || 'string';
            trusted = trusted || false;
            value = this.getFormattedValue(type, value, input, trusted);

            if (!value) {
                return '';
            }

            name = XF.htmlspecialchars(this.options.oldFormat ? name : name.replace('-', '_'));
            nameless = nameless || false;

            if (this.options.oldFormat || this.options.singleOption) {
                if (nameless || this.options.singleOption) {
                    return '"' + value + '"';
                }

                return name + ':' + value;
            }

            return name + '="' + value + '"';
        },

        valueStr: function (value, type, input, trusted) {
            type = type || 'string';
            trusted = trusted || false;
            value = this.getFormattedValue(type, value, input, trusted);

            if (!value) {
                return '';
            }

            return value;
        }
    });

    SV.AdvancedBBCode.EditorButtons.setupCommands = function (config) {
        if (!Array.isArray(config.dialog)) {
            return;
        }

        config.dialog.forEach(function (element) {
            SV.AdvancedBBCode[element] = SV.AdvancedBBCode[element] || {};
            if (!SV.AdvancedBBCode[element].editorButton) {
                SV.AdvancedBBCode[element].editorButton = {
                    init: function () {
                        SV.AdvancedBBCode[element].EditorDialog = XF.extend(XF.EditorDialog, {
                            simpleHtmlSnippet: null,
                            cache: false,
                            $container: null,

                            _init: function (overlay) {
                                this.simpleHtmlSnippet = overlay.$overlay.find('div#bb_code_output_simple_html').html();
                                if (this.simpleHtmlSnippet && this.simpleHtmlSnippet.length) {
                                    this.cache = true;
                                    return;
                                }

                                overlay.$overlay.find('form').submit(XF.proxy(this, 'submit'));
                            },

                            beforeShow: function (overlay, e) {
                                if (this.simpleHtmlSnippet && this.simpleHtmlSnippet.length) {
                                    e.preventDefault();

                                    var self = this;
                                    setTimeout(function () {
                                        self.insertOutput(self.simpleHtmlSnippet);
                                    }, 0);

                                    return;
                                }
                                this.overlay = overlay;

                                this._beforeShow(overlay);
                            },

                            _beforeShow: function (overlay) {
                                this.ed.$el.blur();
                            },

                            _afterShow: function (overlay) {
                                // based on XF.EditorDialogCode
                                var $container = overlay.$container,
                                    ed = this.ed,
                                    selectedText,
                                    bbCodeValue = $container.find('[data-bb-value]:not([data-no-fill])');

                                if (!bbCodeValue.length) {
                                    return;
                                }

                                ed.selection.restore();
                                var $element = bbCodeValue.first();
                                if (!ed.selection.isCollapsed()) {
                                    var isBbCode = !!$element.data('bbValue');
                                    if (isBbCode) {
                                        var html = ed.html.getSelected();
                                        if (html) {
                                            XF.ajax('POST',
                                                XF.canonicalizeUrl('index.php?editor/to-bb-code'),
                                                {html: html},
                                                function (data) {
                                                    $element.val(data.bbCode || selectedText);
                                                }
                                            );
                                        }
                                    } else {
                                        selectedText = ed.selection.text();
                                        $element.val(selectedText);
                                    }
                                }

                                ed.selection.save();
                                $element.focus();
                            },

                            submit: function (e) {
                                e.preventDefault();

                                var $targetForm = $(e.target).closest('form'),
                                    outputInput = $targetForm.find('[name="bb_code_output"]'),
                                    value = outputInput.val(),
                                    respectCursor = outputInput.data('bb-code-respect-cursor') === 'true';

                                if (value) {
                                    this.insertOutput(value, outputInput, respectCursor);
                                }

                                this.overlay.hide();
                            },

                            insertOutput: function (value, outputInput = null, respectCursor = false) {
                                var ed = this.ed;

                                ed.selection.restore();
                                ed.undo.saveStep();
                                if (respectCursor) {
                                    var html =
                                        outputInput.data('bb-code-start')
                                        + '<div class="xf-froala-marker"></div>'
                                        + outputInput.data('bb-code-end');
                                    ed.html.insert(html);
                                    var $marker = ed.$el.find('.xf-froala-marker');
                                    if ($marker.length !== 0) {
                                        ed.selection.setAfter($marker);
                                        $marker.remove();
                                        ed.selection.restore();
                                    }
                                } else {
                                    ed.html.insert(value);
                                }
                                ed.placeholder.hide();
                                ed.undo.saveStep();
                                XF.EditorHelpers.normalizeAfterInsert(ed);

                                ed.events.trigger("charCounter.update");
                            },
                        });
                    },

                    callback: function () {
                        XF.EditorHelpers.loadDialog(this, element);
                    }
                };
            }
            SV.AdvancedBBCode[element].editorButton.init();
            XF.EditorHelpers.dialogs[element] = new SV.AdvancedBBCode[element].EditorDialog(element);

            if ($.FE.COMMANDS['xfCustom_' + element]) {
                $.FE.COMMANDS['xfCustom_' + element].callback = SV.AdvancedBBCode[element].editorButton.callback;
            }
        });
    };

    SV.AdvancedBBCode.EditorFirstStart.push(function (editorConfig) {
        SV.AdvancedBBCode.EditorButtons.setupCommands(editorConfig);
    });

    XF.Element.register('svBbCode-dialog-form', 'SV.AdvancedBBCode.DialogForm');
})(jQuery, window, document);
