<?php

namespace SV\AdvancedBbCodesPack\BbCode\Tag;

use function array_filter;
use function array_map;
use function count;
use function explode;
use function in_array;
use function is_string;
use function strtolower;

/**
 * A BB code renderer for rendering children tags based on usergroup membership
 *
 * @package SV\AdvancedBbCodesPack\BbCode\Tag
 */
class UserGroupConditional extends AbstractTag
{
    use TagWithSingleOptionTrait;

    public static function get(\XF\BbCode\Renderer\AbstractRenderer $renderer, string $rendererType): self
    {
        $app = \XF::app();
        $className = $app->extendClass(static::class);
        /** @var static $obj */
        $obj = new $className($app, $renderer, $rendererType);

        return $obj;
    }

    /**
     * @param array                $tagChildren
     * @param null|string|string[] $tagOption
     * @param array                $tag
     * @param array                $options
     * @return string|\XF\PreEscaped
     */
    public function render(array $tagChildren, $tagOption, array $tag, array $options)
    {
        $ids = $this->getIdsFromTagOption($tagOption);
        $tagName = strtolower($tag['tag']);
        if ($this->isRenderable($tagName, $ids))
        {
            return $this->renderer->renderSubTree($tagChildren, $options);
        }

        return $this->templater->renderTemplate('public:sv_bbcode_usergroup_not_allowed');
    }

    /**
     * @param null|string|string[] $tagOption
     * @return array<int>
     */
    public function getIdsFromTagOption($tagOption): array
    {
        if (!is_string($tagOption))
        {
            $tagOption = '';
        }
        return array_filter(array_map('\intval', explode(',', $tagOption)));
    }

    /** @noinspection SpellCheckingInspection */
    public function getTags(): array
    {
        return [
            'guest',
            'registered',
            'isuserid',
            'isnotuserid',
            'ismemberofusergroup',
            'isnotmemberofusergroup',
        ];
    }

    /** @noinspection SpellCheckingInspection */
    public function isRenderable(string $tagName, array $ids): bool
    {
        $visitor = \XF::visitor();
        $userId = (int)$visitor->user_id;
        switch ($tagName)
        {
            case 'guest':
                return $userId === 0;
            case 'registered':
                return $userId !== 0;
            case 'isuserid':
                return in_array($userId, $ids, true);
            case 'isnotuserid':
                return !in_array($userId, $ids, true);
            case 'ismemberofusergroup':
                return count($ids) !== 0 && $visitor->isMemberOf($ids);
            case 'isnotmemberofusergroup':
                return count($ids) !== 0 && !$visitor->isMemberOf($ids);
            default:
                return false;
        }
    }
}