<?php

namespace SV\AdvancedBbCodesPack\SimilarContent;

use SV\ElasticSearchEssentials\Entity\ISimilarContent;
use SV\ElasticSearchEssentials\Entity\ISimilarContentMetaData;
use SV\ElasticSearchEssentials\Globals;
use XF\Http\Request;
use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\ArrayCollection;
use XF\Repository\User;
use XF\Search\Data\AbstractData;
use XF\Search\Query\Query;
use function json_encode;
use function md5;

class SimilarContentKeywords implements ISimilarContent, ISimilarContentMetaData
{
    /** @var string */
    public $contentType;
    /** @var array */
    public $criteria;
    /**  @var AbstractData */
    public $handler;
    /** @var Request */
    public $request;
    /** @var callable|null */
    public $onPrepare;
    /** @var array */
    public $options = [];

    protected function __construct(AbstractData $handler, string $contentType, array $criteria, Request $request, array $options)
    {
        $this->handler = $handler;
        $this->contentType = $contentType;
        $this->criteria = $criteria;
        $this->request = $request;
        $this->options = $options;
    }

    public static function factory(AbstractData $handler, string $contentType, array $criteria, Request $request, array $options): self
    {
        $class = static::class;
        $class = \XF::extendClass($class);
        return new $class($handler, $contentType, $criteria, $request, $options);
    }

    public function getEntityContentType(): string
    {
        return $this->contentType;
    }

    public function canViewSimilarContents(&$error = null): bool
    {
        return true;
    }

    public function getKeywordsForSimilarContent(): string
    {
        return $this->criteria['keywords'] ?? '';
    }

    public function getCacheIdForSimilarContents(): string
    {
        $permissionCombinationId  = \XF::visitor()->permission_combination_id ?: 1;
        return 'ESESimilar_bbcode_' . md5(json_encode($this->criteria)) . '_' . $permissionCombinationId;
    }

    public function applySimilarContentMetadataToQuery(Query $query): void
    {
        /** @var User $userRepo */
        $userRepo = \XF::app()->repository('XF:User');
        $guestUser = $userRepo->getGuestUser(null, function (array $guestData): array {
            $guestData['permission_combination_id'] = \XF::visitor()->permission_combination_id;

            return $guestData;
        });

        /** @var AbstractCollection $similarContents */
        \XF::asVisitor($guestUser, function () use ($query) {
            Globals::$falsifyIsModerator = false;
            Globals::$shimSearchDefaultGlobalPerms = false;
            try
            {
                $urlConstraints = [];
                $query->forTypeHandler($this->handler, $this->request, $urlConstraints);
                $query->inType($this->contentType);

                foreach ($this->handler->getTypePermissionConstraints($query, true) AS $constraint)
                {
                    $query->withMetadata($constraint);
                }
            }
            finally
            {
                Globals::$falsifyIsModerator = null;
                Globals::$shimSearchDefaultGlobalPerms = null;
            }
        }) ?? new ArrayCollection([]);

        $onPrepare = $this->onPrepare;
        if ($onPrepare !== null)
        {
            $onPrepare($this, $query);
        }
    }
}