<?php

namespace SV\AdvancedBbCodesPack\XF\BbCode\ProcessorAction;

use SV\AdvancedBbCodesPack\BbCode\Tag\UserGroupConditional;
use XF\BbCode\ProcessorAction\FiltererHooks;
use function strtolower;

/**
 * @extends \XF\BbCode\ProcessorAction\StripQuotes
 */
class StripQuotes extends XFCP_StripQuotes
{
    /** @var UserGroupConditional|null */
    protected $svUserGroupConditional = null;

    public function addFiltererHooks(FiltererHooks $hooks)
    {
        parent::addFiltererHooks($hooks);

        if ($this->svUserGroupConditional === null)
        {
            $bbCodeCleaner = \XF::app()->bbCode()->renderer('bbCodeClean');
            $this->svUserGroupConditional = UserGroupConditional::get($bbCodeCleaner, 'html');
        }

        foreach ($this->svUserGroupConditional->getTags() as $tag)
        {
            $hooks->addTagHook($tag, 'filterSvUserGroupTag');
        }
    }

    public function filterSvUserGroupTag(array $tag, array $options): ?string
    {
        $conditional = $this->svUserGroupConditional;
        $ids = $conditional->getIdsFromTagOption($options);
        $tagName = strtolower($tag['tag']);

        if ($conditional->isRenderable($tagName, $ids))
        {
            // keep
            return true;
        }

        // remove everything within
        return '';
    }
}