<?php
/**
 * @noinspection PhpMissingReturnTypeInspection
 */

namespace SV\AdvancedBbCodesPack\XF\BbCode\Renderer;

use SV\AdvancedBbCodesPack\Repository\BbCode as BbCodeRepo;
use XF\Str\Formatter;
use XF\Template\Templater;
use function intval;
use function preg_replace;

/**
 * @Extends \XF\BbCode\Renderer\Html
 */
class HtmlPatch extends XFCP_HtmlPatch
{
    /** @var bool */
    protected $svShimTagHeading = false;
    /**  @var bool */
    protected $svNormalizeCharacters = false;

    public function __construct(Formatter $formatter, Templater $templater)
    {
        parent::__construct($formatter, $templater);

        $this->svNormalizeCharacters = (\XF::options()->svNormalizeQuoteCharactersOnRender ?? false);
    }

    public function addDefaultTags()
    {
        parent::addDefaultTags();

        $this->svShimTagHeading = !BbCodeRepo::get()->isStockHeadingEnabled();
    }

    public function renderTagHeading(array $children, $option, array $tag, array $options)
    {
        if ($this->svShimTagHeading)
        {
            $type = $option ? intval($option) : 0;
            $newTag = $this->getHeadingTagMap($type);

            if ($this->rules->getTag($newTag) !== null)
            {
                $tag['tag'] = $newTag;
                $tag['option'] = '';

                return parent::renderTag($tag, $options);
            }
        }

        return parent::renderTagHeading($children, $option, $tag, $options);
    }


    public function wrapHtml($open, $inner, $close, $option = null)
    {
        // Note; $option isn't ever used, and contains a potential security issue
        //if ((\XF::$versionId < 2021100 && \XF::$versionId > 2020000) || (\XF::$versionId < 2011300))
        $option = null;

        return parent::wrapHtml($open, $inner, $close, $option);
    }

    public function filterString($string, array $options)
    {
        if ($this->svNormalizeCharacters)
        {
            $string1 = preg_replace('/[\x{2018}-\x{201B}]/ui', '\'', $string);
            if ($string1 !== null)
            {
                $string = $string1;
            }
            $string1 = preg_replace('/[\x{201C}-\x{201F}\x{2E42}\x{301D}-\x{301F}]/ui', '"', $string);
            if ($string1 !== null)
            {
                $string = $string1;
            }
            $string1 = preg_replace('/[\x{2000}-\x{200A}\x{202F}\x{205F}\x{3000}]/ui', ' ', $string);
            if ($string1 !== null)
            {
                $string = $string1;
            }
        }

        return parent::filterString($string, $options);
    }
}