<?php

namespace SV\AdvancedBbCodesPack\XenAddons\AMS\Entity;

use XF\Util\Arr;
use function gmdate;
use function is_callable;

/**
 * @Extends \XenAddons\AMS\Entity\ArticleItem
 */
class ArticleItemPatch extends XFCP_ArticleItemPatch
{
    protected function getLdSnippet(string $message, int $length = null): string
    {
        if ($length === null)
        {
            $length = 250;
        }

        return \XF::app()->stringFormatter()->snippetString($message, $length, ['stripBbCode' => true]);
    }

    public function getLdStructuredData(int $page = 1, array $extraData = []): array
    {
        if (is_callable([parent::class, 'getLdStructuredData']))
        {
            return parent::getLdStructuredData($page, $extraData);
        }

        $router = $this->app()->router('public');
        $templater = \XF::app()->templater();
        $output = [
            '@context'            => 'https://schema.org',
            '@type'               => 'CreativeWorkSeries',
            '@id'                 => $router->buildLink('canonical:ams', $this),
            'name'                => $this->title,
            'headline'            => $this->meta_title ?: $this->title,
            'alternativeHeadline' => $this->og_title ?: $this->title,
            'description'         => $this->getLdSnippet($this->meta_description ?: $this->description ?: $this->message),
            'thumbnailUrl'        => ($this->cover_image_id && $this->CoverImage)
                ? $this->CoverImage->getThumbnailUrlFull()
                : $templater->func('base_url', [$this->Category->content_image_url]),
            'dateCreated'         => gmdate('c', $this->publish_date),
            'dateModified'        => gmdate('c', $this->last_update),
            'author'              => [
                '@type' => 'Person',
                'name'  => $this->User->Profile->xa_ams_author_name
                           ?: $this->User->username
                           ?: $this->username,
            ]
        ];

        if ($this->rating_count)
        {
            $output['aggregateRating'] = [
                "@type"       => 'AggregateRating',
                "ratingCount" => $this->rating_count,
                "ratingValue" => $this->rating_avg,
            ];
        }

        if ($this->hasViewableDiscussion())
        {
            $output['discussionUrl'] = $router->buildLink('canonical:threads', $this->Discussion);
        }

        return Arr::filterNull($output, true);
    }
}