<?php

namespace SV\AdvancedBbCodesPack\XenAddons\AMS\Entity;

use XF\Util\Arr;
use function gmdate;
use function is_callable;

/**
 * @Extends \XenAddons\AMS\Entity\ArticlePage
 *
 * @property string $meta_title
 * @property string $og_title
 * @property string $description
 */
class ArticlePagePatch extends XFCP_ArticlePagePatch
{
    protected function getLdSnippet(string $message, int $length = null): string
    {
        if ($length === null)
        {
            $length = 250;
        }

        return \XF::app()->stringFormatter()->snippetString($message, $length, ['stripBbCode' => true]);
    }

    public function getLdStructuredData(int $page = 1, array $extraData = []): array
    {
        if (is_callable([parent::class, 'getLdStructuredData']))
        {
            return parent::getLdStructuredData($page, $extraData);
        }

        $router = $this->app()->router('public');
        $templater = \XF::app()->templater();
        $article = $this->Article;

        $output = [
            '@context'            => 'https://schema.org',
            '@type'               => 'CreativeWorkSeries',
            '@id'                 => $router->buildLink('canonical:ams/page', $this),
            'name'                => $this->title,
            'headline'            => ($article->meta_title ?: $this->title) . ' | ' . ($this->meta_title ?: $this->title),
            'alternativeHeadline' => ($article->og_title ?: $this->title) . ' | ' . ($this->og_title ?: $this->title),
            'description'         => $this->getLdSnippet($this->meta_description ?: $this->description ?: $this->message),
            'thumbnailUrl'        => ($this->cover_image_id && $this->CoverImage)
                ? ($this->cover_image_id && $this->CoverImage)
                : (($article->cover_image_id && $article->CoverImage)
                    ? $article->CoverImage->getThumbnailUrlFull()
                    : $templater->func('base_url', [$article->Category->content_image_url])),
            'dateCreated'         => gmdate('c', $this->create_date),
            'dateModified'        => gmdate('c', $this->edit_date),
            'author'              => [
                '@type' => 'Person',
                'name'  => $this->User->Profile->xa_ams_author_name
                           ?: $this->User->username
                           ?: $this->username,
            ]
        ];

        if ($article->hasViewableDiscussion())
        {
            $output['discussionUrl'] = $router->buildLink('canonical:threads', $article->Discussion);
        }

        return Arr::filterNull($output, true);
    }
}